<?php

namespace Drupal\druhels;

class ArrayHelper {

  /**
   * Remove empty elements from array.
   *
   * @param array $array Source array
   */
  public static function removeEmptyElements(array $array, $recursive = FALSE): array {
    foreach ($array as $key => $value) {
      if ($value && is_array($value)) {
        $array[$key] = self::removeEmptyElements($array[$key]);
      }
      if (!$value) {
        unset($array[$key]);
      }
    }

    return $array;
  }

  /**
   * Rename keys to column value.
   *
   * @param array $array Source array
   * @param mixed $column_key Column key
   */
  public static function renameKeysToColumnValue(array $array, $column_key): array {
    $new_array = array();
    foreach ($array as $key => $values) {
      if (isset($values[$column_key])) {
        $new_array[$values[$column_key]] = $values;
      }
    }
    return $new_array;
  }

  /**
   * Insert element before/after key.
   *
   * @param array $source_array Input array
   * @param integer|string $source_array_key Array key
   * @param array $inserted_element Inserted element
   * @param string $insert_type Insertion type 'before' or 'after'
   */
  public static function insert(array $source_array, $source_array_key, array $inserted_element, $insert_type): array {
    $key_position = array_search($source_array_key, array_keys($source_array), TRUE);

    if ($key_position !== FALSE) {
      $offset = ($insert_type == 'after') ? 1 : 0;
      $array_before = array_slice($source_array, 0, $key_position + $offset, TRUE);
      $array_after = array_slice($source_array, $key_position + $offset, count($source_array), TRUE);
      $array = $array_before + $inserted_element + $array_after;
    }
    else {
      $array = $source_array + $inserted_element;
    }

    return $array;
  }

  /**
   * Insert element before key.
   *
   * @param array $source_array Input array
   * @param integer|string $source_array_key Array key
   * @param array $inserted_element Inserted element
   */
  public static function insertBefore(array $source_array, $source_array_key, array $inserted_element): array {
    return self::insert($source_array, $source_array_key, $inserted_element, 'before');
  }

  /**
   * Insert element after key.
   *
   * @param array $source_array Input array
   * @param integer|string $source_array_key Array key
   * @param array $inserted_element Inserted element
   */
  public static function insertAfter(array $source_array, $source_array_key, array $inserted_element): array {
    return self::insert($source_array, $source_array_key, $inserted_element, 'after');
  }

  /**
   * Insert element after element with specific value.
   *
   * @param array $source_array Input array
   * @param mixed $source_element_value Searched element value
   * @param array $inserted_element Inserted element
   */
  public static function insertAfterValue(array $source_array, $source_element_value, array $inserted_element): array {
    $source_array_key = array_search($source_element_value, $source_array);
    if ($source_array_key !== FALSE) {
      return self::insertAfter($source_array, $source_array_key, $inserted_element);
    }
    return array_merge($source_array, $inserted_element);
  }

  /**
   * Move element before key.
   *
   * @param array $array Input array
   * @param mixed $key Element key to move
   * @param mixed $before_key Element key before
   */
  public static function moveBefore(array $array, $key, $before_key): array {
    if (array_key_exists($before_key, $array)) {
      $element = $array[$key];
      unset($array[$key]);
      $array = self::insertBefore($array, $before_key, [$key => $element]);
    }

    return $array;
  }

  /**
   * Rename key.
   *
   * @param array $array Input array
   * @param mixed $old_key Old key name
   * @param mixed $new_key New key name
   */
  public static function renameKey(array $array, $old_key, $new_key): array {
    if (isset($array[$old_key])) {
      $array = self::insertAfter($array, $old_key, array($new_key => $array[$old_key]));
      unset($array[$old_key]);
    }
    return $array;
  }

  /**
   * Remove elements from array by keys.
   *
   * @param array $array Input array
   * @param array $keys Array of keys for remove
   */
  public static function removeElementsByKey(array $array, $keys): array {
    foreach ($keys as $key) {
      if (array_key_exists($key, $array)) {
        unset($array[$key]);
      }
    }

    return $array;
  }

  /**
   * Remove elements from $array which one keys are not in $keys.
   *
   * @param array $array Input array
   * @param array $keys Array of keys
   */
  public static function removeExtraElements(array $array, $keys): array {
    return array_intersect_key($array, array_fill_keys($keys, NULL));
  }

  /**
   * Return text lines in array.
   *
   * @param string $text Text
   */
  public static function getTextLines($text, $remove_empty = TRUE): array {
    $lines = preg_split("/\r\n|\n|\r/", $text);

    if ($remove_empty) {
      $lines = self::removeEmptyElements($lines);
    }

    return $lines;
  }

  /**
   * Search in two-dimensional array and return element key.
   *
   * @param array $array Two-dimensional array
   * @param mixed $key Second dimension key
   * @param mixed $value Search value
   * @return mixed|FALSE Key
   */
  public static function searchInTwodimArray(array $array, $key, $value, $strict = FALSE) {
    foreach ($array as $array_key => $second_array) {
      if (array_key_exists($key, $second_array)) {
        if ($strict) {
          if ($second_array[$key] == $value) {
            return $array_key;
          }
        }
        elseif ($second_array[$key] === $value) {
          return $array_key;
        }
      }
    }

    return FALSE;
  }

  /**
   * Return first key.
   *
   * @return mixed
   */
  public static function getFirstKey(array $array) {
    reset($array);
    return key($array);
  }

  /**
   * Return last key.
   *
   * @return mixed
   */
  public static function getLastKey(array $array) {
    end($array);
    return key($array);
  }

  /**
   * Sort array by second array values order.
   */
  public static function sortBySecondArrayValues(array $array, array $second_array): array {
    $sorted_array = array();

    foreach ($second_array as $value) {
      if (array_key_exists($value, $array)) {
        $sorted_array[$value] = $array[$value];
      }
    }

    $sorted_array += $array;

    return $sorted_array;
  }

  /**
   * Sort array by second array keys order.
   */
  public static function sortBySecondArrayKeys(array $array, array $second_array): array {
    $sorted_array = array();

    foreach (array_keys($second_array) as $key) {
      if (array_key_exists($key, $array)) {
        $sorted_array[$key] = $array[$key];
      }
    }

    $sorted_array += $array;

    return $sorted_array;
  }

  /**
   * Implode multidimensional array.
   */
  public static function implodeMultiArray(array $array, $root_glue, $second_glue): string {
    foreach ($array as $key => $values) {
      if (is_array($values)) {
        $array[$key] = implode($second_glue, $values);
      }
    }

    return implode($root_glue, $array);
  }

  /**
   * Return TRUE if all keys exists in $array.
   *
   * @param array $array
   * @param array $keys
   */
  public static function keysExists($array, $keys): bool {
    foreach ($keys as $key) {
      if (!array_key_exists($key, $array)) {
        return FALSE;
      }
    }

    return TRUE;
  }

}
