<?php

namespace Drupal\druhels;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_product\Entity\ProductInterface;
use Drupal\commerce_store\Entity\Store;
use Drupal\Core\Session\AccountInterface;

class CommerceHelper {

  /**
   * Return current user carts.
   *
   * @return OrderInterface[]
   */
  public static function getCarts(AccountInterface $account = NULL) {
    /** @see \Drupal\commerce_cart\Plugin\Block\CartBlock::build() */
    $carts = \Drupal::service('commerce_cart.cart_provider')->getCarts($account); /** @var OrderInterface[] $carts */

    if ($carts) {
      return array_filter($carts, function ($cart) {
        /** @var OrderInterface $cart */
        return $cart->hasItems() && $cart->cart->value;
      });
    }

    return [];
  }

  /**
   * Return carts total price.
   */
  public static function getCartsTotalPrice(AccountInterface $account = NULL): Price {
    $carts_total_price = new Price(0, self::getCurrentStoreCurrencyCode());

    if ($carts = self::getCarts($account)) {
      foreach ($carts as $cart) {
        if ($cart_total_price = $cart->getTotalPrice()) {
          $carts_total_price = $carts_total_price->add($cart_total_price);
        }
      }
    }

    return $carts_total_price;
  }

  /**
   * Return current store currency code.
   */
  public static function getCurrentStoreCurrencyCode(): string {
    $current_store = \Drupal::service('commerce_store.current_store')->getStore(); /** @var Store $current_store */
    return $current_store->getDefaultCurrencyCode();
  }

  /**
   * Return order total quantity.
   */
  public static function getOrderTotalQuantity(OrderInterface $order): int {
    $quantity = 0;

    foreach ($order->getItems() as $order_item) {
      $quantity += (int)$order_item->getQuantity();
    }

    return $quantity;
  }

  /**
   * Return carts items total quantity.
   */
  public static function getCartsTotalQuantity(): int {
    $quantity = 0;

    foreach (self::getCarts() as $cart) {
      $quantity += self::getOrderTotalQuantity($cart);
    }

    return $quantity;
  }

  /**
   * Return formatted price.
   *
   * @param Price|array
   */
  public static function formatPrice($price): string {
    if (empty($price)) {
      return '';
    }

    if ($price instanceof Price) {
      $price = $price->toArray();
    }
    if (is_array($price) && isset($price['currency_code']) && isset($price['number'])) {
      $currency_formatter = \Drupal::service('commerce_price.currency_formatter');
      return $currency_formatter->format($price['number'], $price['currency_code']);
    }
  }

  /**
   * Return TRUE if current page is product page.
   */
  public static function isProductPage($product_type = NULL): bool {
    static $current_product_type;

    if ($current_product_type === NULL) {
      $route_match = \Drupal::routeMatch();
      if ($route_match->getRouteName() == 'entity.commerce_product.canonical') {
        $current_product_type = $route_match->getParameter('commerce_product')->bundle();
      }
      else {
        $current_product_type = FALSE;
      }
    }

    return $product_type ? ($current_product_type == $product_type) : (bool)$current_product_type;
  }

  /**
   * Return current product id.
   */
  public static function getCurrentProductId(): ?int {
    return \Drupal::routeMatch()->getRawParameter('commerce_product');
  }

  /**
   * Return current product.
   */
  public static function getCurrentProduct(): ?ProductInterface {
    if ($product_id = self::getCurrentProductId()) {
      return Product::load($product_id);
    }
    return NULL;
  }

  /**
   * Return product render array.
   */
  public static function productView(ProductInterface $product, $view_mode = 'full'): array {
    return \Drupal::entityTypeManager()->getViewBuilder('commerce_product')->view($product, $view_mode);
  }

}
