<?php

namespace Drupal\druhels;

use Drupal\Component\Utility\Timer;
use Drupal\Core\Controller\TitleResolverInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Symfony\Cmf\Component\Routing\RouteObjectInterface;
use Symfony\Component\Routing\Route;

class DrupalHelper {

  /**
   * Return current page title.
   *
   * @return string|array
   */
  public static function getCurrentPageTitle() {
    $request = \Drupal::request();
    $route = $request->attributes->get(RouteObjectInterface::ROUTE_OBJECT);
    $title_resolver = \Drupal::service('title_resolver'); /** @var TitleResolverInterface $title_resolver */
    $page_title = $title_resolver->getTitle($request, $route);

    if (is_array($page_title) && isset($page_title['#markup'])) {
      return $page_title['#markup'];
    }
    return $page_title;
  }

  /**
   * Set page title.
   *
   * @param string $title Page title
   * @param boolean $unset_title_callback Set TRUE if must unset route title callback
   *
   * @return null
   */
  public static function setCurrentPageTitle($title, $unset_title_callback = FALSE) {
    $current_route = \Drupal::request()->attributes->get(RouteObjectInterface::ROUTE_OBJECT); /** @var Route $current_route */
    $current_route->setDefault('_title', $title);

    if ($unset_title_callback) {
      $current_route->setDefault('_title_callback', NULL);
    }
  }

  /**
   * Return current system path without trailing slashes.
   *
   * @return string
   */
  public static function getCurrentSystemPath() {
    $current_system_path = \Drupal::service('path.current')->getPath();
    return ltrim($current_system_path, '/');
  }

  /**
   * Return current path alias without trailing slashes.
   *
   * @return string
   */
  public static function getCurrentPathAlias() {
    $current_system_path = self::getCurrentSystemPath();
    $current_path_alias = \Drupal::service('path.alias_manager')->getAliasByPath('/' . $current_system_path);
    return ltrim($current_path_alias, '/');
  }

  /**
   * Return current route name.
   *
   * @return string
   */
  public static function getCurrentRouteName() {
    static $current_route_name;

    if ($current_route_name === NULL) {
      $current_route_name = \Drupal::routeMatch()->getRouteName();
    }

    return $current_route_name;
  }

  /**
   * Return TRUE if current route is admin route.
   *
   * @return boolean
   */
  public static function isAdminRoute() {
    static $cache = NULL;

    if ($cache === NULL) {
      $cache = \Drupal::service('router.admin_context')->isAdminRoute();
    }

    return $cache;
  }

  /**
   * Return TRUE if current page is front.
   *
   * @return boolean
   */
  public static function isFrontPage() {
    static $cache = NULL;

    if ($cache === NULL) {
      $cache = \Drupal::service('path.matcher')->isFrontPage();
    }

    return $cache;
  }

  /**
   * Start timer.
   *
   * @return null
   */
  public static function timerStart($name = 'druhels') {
    Timer::start($name);
  }

  /**
   * Stop timer and show result.
   *
   * @return null
   */
  public static function timerStop($name = 'druhels', $show_result = TRUE) {
    Timer::stop($name);

    if ($show_result) {
      $duration = Timer::read($name);
      \Drupal::messenger()->addMessage($duration . ' ms / ' . number_format($duration / 1000, 5) . ' sec');
    }
  }

  /**
   * Return default langcode (language id).
   *
   * @return string
   */
  public static function getDefaultLangcode() {
    static $default_language_id;

    if ($default_language_id === NULL) {
      $default_language_id = \Drupal::languageManager()->getDefaultLanguage()->getId();
    }

    return $default_language_id;
  }

  /**
   * Return current langcode (language id).
   *
   * @return string
   */
  public static function getCurrentLangcode() {
    static $current_language_id;

    if ($current_language_id === NULL) {
      $current_language_id = \Drupal::languageManager()->getCurrentLanguage()->getId();
    }

    return $current_language_id;
  }

  /**
   * Send mail without need implements hook_mail().
   *
   * @return array
   */
  public static function sendMail($to, $subject, $message, $key = 'druhels', $langcode = NULL, $params = []) {
    if (!$langcode) {
      $langcode = \Drupal::languageManager()->getDefaultLanguage()->getId();
    }

    $mail_service = \Drupal::service('plugin.manager.mail'); /* @var MailManagerInterface $mail_service */

    $params['subject'] = $subject;
    $params['body'] = $message;

    return $mail_service->mail('druhels', $key, $to, $langcode, $params);
  }

  /**
   * Return breakpoint media query.
   *
   * @return string
   */
  public static function getBreakpointMediaQuery($group, $breakpoint_name) {
    \Drupal::service('breakpoint.manager')->getBreakpointsByGroup($group)[$breakpoint_name]->getMediaQuery();
  }

  /**
   * Return site mail address.
   */
  public static function getSiteMail(): string {
    return \Drupal::config('system.site')->get('mail');
  }

}
