<?php

namespace Drupal\druhels;

use BackupMigrate\Core\Translation\TranslatableInteface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\TypedData\TranslatableInterface;
use Drupal\field\Entity\FieldStorageConfig;

class EntityHelper {

  /**
   * Return field label.
   */
  public static function getFieldLabel($entity_type, $bundle_name, $field_name) {
    $entity_field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManagerInterface $entity_field_manager */
    return $entity_field_manager->getFieldDefinitions($entity_type, $bundle_name)[$field_name]->getLabel();
  }

  /**
   * Return list field allowed values.
   *
   * @return array
   */
  public static function getListFieldAllowedValues($entity_type, $entity_bundle, $field_name) {
    $field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManagerInterface $field_manager */
    $field_definition = $field_manager->getFieldDefinitions($entity_type, $entity_bundle)[$field_name];
    $field_storage_definition = $field_definition->getFieldStorageDefinition();
    return options_allowed_values($field_storage_definition);
  }

  /**
   * Return value label for List field.
   */
  public static function getListFieldValueLabel(FieldItemListInterface $field_items, $delta = 0) {
    $allowed_values = options_allowed_values($field_items->getFieldDefinition()->getFieldStorageDefinition(), $field_items->getEntity());
    $value = $field_items->value;
    return ($value !== NULL && isset($allowed_values[$value])) ? $allowed_values[$value] : $value;
  }

  /**
   * Return field main values.
   *
   * @return array
   */
  public static function getFieldMainValues(FieldItemListInterface $items) {
    if ($values = $items->getValue()) {
      $main_property_name = $items->first()->mainPropertyName() ?: key(current($values));
      return array_column($values, $main_property_name);
    }
    return [];
  }

  /**
   * Return TRUE if bundle has field.
   *
   * @return boolean
   */
  public static function bundleHasField($entity_type, $bundle, $field_name) {
    $field_storage = FieldStorageConfig::loadByName($entity_type, $field_name);
    return $field_storage && in_array($bundle, $field_storage->getBundles());
  }

  /**
   * Return bundle field definitions by field type.
   *
   * @param string $entity_type_id
   * @param string $bundle
   * @param string $field_type
   *
   * @return FieldDefinitionInterface[]
   */
  public static function getFieldDefinitionsByType($entity_type_id, $bundle, $field_type) {
    $entity_field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManager $entity_field_manager */
    $field_definitions = $entity_field_manager->getFieldDefinitions($entity_type_id, $bundle);
    return array_filter($field_definitions, function ($field_definition) use ($field_type) {
      /** @var FieldDefinitionInterface $field_definition */
      return $field_definition->getType() == $field_type;
    });
  }

  /**
   * Return entity build array.
   *
   * @param EntityInterface|integer $entity - Entity object or id.
   *
   * @return array Entity render array.
   */
  public static function view(EntityInterface $entity, $view_mode = 'full') {
    $entity_type = $entity->getEntityTypeId();
    return \Drupal::entityTypeManager()->getViewBuilder($entity_type)->view($entity, $view_mode);
  }

  /**
   * Return entity build array with fields.
   *
   * @return array
   */
  public static function renderEntityFields(EntityInterface $entity, $view_mode = 'full') {
    $build = self::view($entity, $view_mode);

    foreach ($build['#pre_render'] as $callable) {
      $build = call_user_func($callable, $build);
    }

    unset($build['#pre_render']);

    return $build;
  }

  /**
   * Delete multiple entities.
   *
   * @param integer[] $entity_ids
   * @param string $entity_type Entity type id
   */
  public static function deleteMultiple(array $entity_ids, $entity_type) {
    $entity_storage = \Drupal::entityTypeManager()->getStorage($entity_type);
    $entities = $entity_storage->loadMultiple($entity_ids);
    return $entity_storage->delete($entities);
  }

  /**
   * Return TRUE if entity field value changed.
   */
  public static function fieldHasAffectingChanges($entity, $field_name) {
    $original_entity = $entity->original;
    $field_items = $entity->get($field_name);
    $original_field_items = $original_entity->get($field_name);

    return $field_items->hasAffectingChanges($original_field_items, $field_items->getLangcode());
  }

  /**
   * Return TRUE if list field has certain value.
   *
   * @return boolean
   */
  public static function listFieldHasValue($items, $value) {
    return in_array($value, self::getFieldMainValues($items));
  }

  /**
   * Return content entity types.
   *
   * @return ContentEntityTypeInterface[]
   */
  public static function getContentEntityTypes() {
    return array_filter(\Drupal::entityTypeManager()->getDefinitions(), function (EntityTypeInterface $entity_type) {
      return $entity_type instanceof ContentEntityTypeInterface;
    });
  }

  /**
   * Return TRUE if current page is entity canonical page.
   *
   * @return boolean
   */
  public static function isEntityPage($entity_type_id = NULL) {
    if (preg_match('/^entity\.(.+)\.canonical$/', DrupalHelper::getCurrentRouteName(), $matches)) {
      if ($entity_type_id) {
        return $matches[1] == $entity_type_id;
      }

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Return current entity.
   *
   * @return EntityInterface|null
   */
  public static function getCurrentEntity() {
    if (preg_match('/^entity\.(.+)\.canonical$/', DrupalHelper::getCurrentRouteName(), $matches)) {
      $entity_type_id = $matches[1];
      $entity = \Drupal::routeMatch()->getParameter($entity_type_id); /** @var EntityInterface $entity */

      if ($entity && is_numeric($entity)) {
        $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($entity);
      }

      return $entity;
    }
  }

  /**
   * Return translated entity.
   *
   * @return EntityInterface
   */
  public static function getTranslatedEntity(TranslatableInterface $entity, $langcode = NULL) {
    if (!$langcode) {
      $langcode = DrupalHelper::getCurrentLangcode();
    }

    return $entity->hasTranslation($langcode) ? $entity->getTranslation($langcode) : $entity;
  }

}
