<?php

namespace Drupal\druhels;

use Drupal\Core\Url;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\TermInterface;
use Drupal\taxonomy\TermStorageInterface;

class TaxonomyHelper {

  /**
   * Return term by name.
   *
   * @return TermInterface|null
   */
  public static function getTermByName($vocabulary_machine_name, $term_name, $parent_id = NULL) {
    $term_storate = self::getTermStorage();
    $term_properties = [
      'name' => $term_name,
      'vid' => $vocabulary_machine_name,
    ];
    if ($parent_id !== NULL) {
      $term_properties['parent'] = $parent_id;
    }
    $terms = $term_storate->loadByProperties($term_properties);

    return $terms ? current($terms) : NULL;
  }

  /**
   * Create term and return term entity object. If term with this name exists then return it.
   *
   * @return TermInterface
   */
  public static function createTerm($vocabulary_machine_name, $term_name, $parent_id = 0, $properties = []) {
    $term = self::getTermByName($vocabulary_machine_name, $term_name, $parent_id);

    if (!$term) {
      $term = Term::create([
        'name' => $term_name,
        'vid' => $vocabulary_machine_name,
        'parent' => $parent_id,
      ] + $properties);
      $term->save();
    }

    return $term;
  }

  /**
   * Return terms by names hierarchy.
   *
   * @return TermInterface[]
   */
  public static function getTermsByNamesHierarchy($vocabulary_machine_name, array $names) {
    $parent_id = 0;
    $terms = [];

    foreach ($names as $name) {
      if ($current_term = self::getTermByName($vocabulary_machine_name, $name, $parent_id)) {
        $terms[$current_term->id()] = $current_term;
        $parent_id = $current_term->id();
      }
      else {
        return [];
      }
    }

    return $terms;
  }

  /**
   * Create terms by names hierarchy.
   * Example: "TaxonomyHelper::createTermsByNamesHierarchy(['Term 1', 'Term 1.1'])" creates two terms and parent of "Term 1.1" will be "Term 1".
   *
   * @param string $vocabulary_machine_name Vocabulary machine name
   * @param array $names Array of terms names
   *
   * @return TermInterface[]
   */
  public static function createTermsByNamesHierarchy($vocabulary_machine_name, array $names) {
    $terms = [];
    $parent_id = 0;

    foreach ($names as $name) {
      $term = self::createTerm($vocabulary_machine_name, $name, $parent_id);
      $terms[$term->id()] = $term;
      $parent_id = $term->id();
    }

    return $terms;
  }

  /**
   * Return vocabulary terms sorted by weight. It's analogue of $term_storage->loadTree() but array keys is term ids.
   *
   * @return array|TermInterface[]
   */
  public static function getVocabularyTerms($vocabulary_machine_name, $parent_id = 0, $depth = NULL, $load_entities = FALSE) {
    $term_storage = self::getTermStorage();
    $terms = $term_storage->loadTree($vocabulary_machine_name, $parent_id, $depth, $load_entities);
    $terms_formatted = [];

    foreach ($terms as $key => $term) {
      $term_id = $load_entities ? $term->id() : $term->tid;
      $terms_formatted[$term_id] = $term;
    }

    return $terms_formatted;
  }

  /**
   * Return all child terms regardless of depth. It's analogue of $term_storage->loadTree() but array keys is term ids.
   *
   * @return array|TermInterface[]
   */
  public static function getAllChildTerms($vocabulary_machine_name, $term_id, $load_entities = FALSE) {
    return self::getVocabularyTerms($vocabulary_machine_name, $term_id, NULL, $load_entities);
  }

  /**
   * Return all child terms ids regardless of depth.
   *
   * @return integer[]
   */
  public static function getAllChildTermsIds($vocabulary_machine_name, $term_id, $add_current_term_in_result = FALSE) {
    $term_storage = self::getTermStorage();
    $terms = $term_storage->loadTree($vocabulary_machine_name, $term_id);
    $ids = $add_current_term_in_result ? [$term_id] : [];

    foreach ($terms as $term) {
      $ids[] = (int)$term->tid;
    }

    return $ids;
  }

  /**
   * Return direct child terms.
   *
   * @return array|TermInterface[]
   */
  public static function getDirectChildTerms($vocabulary_machine_name, $term_id, $load_entities = FALSE) {
    return self::getVocabularyTerms($vocabulary_machine_name, $term_id, 1, $load_entities);
  }

  /**
   * Return TRUE if term has child terms.
   *
   * @return boolean
   */
  public static function termHasChilds($term_id) {
    static $cache = [];

    if (!isset($cache[$term_id])) {
      $cache[$term_id] = \Drupal::entityQuery('taxonomy_term')
        ->condition('parent', $term_id)
        ->count()
        ->execute() > 0;
    }

    return $cache[$term_id];
  }

  /**
   * Return all parents terms regardless of depth including self term.
   *
   * @return TermInterface[]
   */
  public static function getAllParentsTerms($term_id, $including_self_term = TRUE) {
    $term_storage = self::getTermStorage();
    $term_with_all_parents = $term_storage->loadAllParents($term_id);

    if ($term_with_all_parents && !$including_self_term) {
      unset($term_with_all_parents[$term_id]);
    }
    elseif (!$term_with_all_parents && $including_self_term && ($term = Term::load($term_id))) {
      $term_with_all_parents[$term_id] = $term;
    }

    return array_reverse($term_with_all_parents, TRUE);
  }

  /**
   * Return all parents terms ids regardless of depth with passed term.
   *
   * @return integer[]
   */
  public static function getAllParentsTermsIds($term_id, $including_self_term = TRUE) {
    static $cache = [];

    if (!isset($cache[$term_id])) {
      $cache[$term_id] = array_keys(self::getAllParentsTerms($term_id, TRUE));
    }

    return $including_self_term ? $cache[$term_id] : array_diff($cache[$term_id], [$term_id]);
  }

  /**
   * Return root parent term. Return self term if he doesn't have parents.
   *
   * @return TermInterface|null
   */
  public static function getRootParentTerm($term_id) {
    $term_with_all_parents = self::getAllParentsTerms($term_id);
    return $term_with_all_parents ? current($term_with_all_parents) : NULL;
  }

  /**
   * Return root parent term id. Return self term if he doesn't have parents.
   *
   * @return integer|null
   */
  public static function getRootParentTermId($term_id) {
    $root_parent_term = self::getRootParentTerm($term_id);
    return $root_parent_term ? $root_parent_term->id() : NULL;
  }

  /**
   * Return TRUE if current page is term page.
   *
   * @param string|string[] $vocabulary_machine_name Vocabulary machine name or names array
   *
   * @return boolean
   */
  public static function isTermPage($vocabulary_machine_name = NULL) {
    static $current_vocabulary_machine_name;

    if ($current_vocabulary_machine_name === NULL) {
      if (DrupalHelper::getCurrentRouteName() == 'entity.taxonomy_term.canonical') {
        $term = self::getCurrentTerm();
        $current_vocabulary_machine_name = $term->bundle();
      }
      else {
        $current_vocabulary_machine_name = FALSE;
      }
    }

    if ($current_vocabulary_machine_name) {
      if ($vocabulary_machine_name) {
        if (!is_array($vocabulary_machine_name)) {
          $vocabulary_machine_name = [$vocabulary_machine_name];
        }
        return in_array($current_vocabulary_machine_name, $vocabulary_machine_name);
      }

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Return current term id from url.
   *
   * @return integer|null
   */
  public static function getCurrentTermId() {
    static $current_term_id;

    if ($current_term_id === NULL) {
      $current_term_id = \Drupal::routeMatch()->getRawParameter('taxonomy_term') ?: FALSE;
    }

    return $current_term_id ?: NULL;
  }

  /**
   * Return current term object from url.
   *
   * @return TermInterface|null
   */
  public static function getCurrentTerm() {
    if ($term_id = self::getCurrentTermId()) {
      return Term::load($term_id);
    }
  }

  /**
   * Return term id by term properties.
   *
   * @return integer|null
   */
  public static function getTermIdByProperties(array $properties) {
    $term_query = \Drupal::entityQuery('taxonomy_term');
    $term_query->range(0, 1);

    foreach ($properties as $field_name => $field_value) {
      $term_query->condition($field_name, $field_value);
    }

    $result = $term_query->execute();

    return $result ? current($result) : NULL;
  }

  /**
   * Return term url object.
   *
   * @return Url
   */
  public static function getTermUrl($tid, $options = []) {
    return Url::fromRoute('entity.taxonomy_term.canonical', ['taxonomy_term' => $tid], $options);
  }

  /**
   * Return term storage.
   *
   * @return TermStorageInterface
   */
  public static function getTermStorage() {
    return \Drupal::entityTypeManager()->getStorage('taxonomy_term');
  }

}
